@@ -561,7 +561,7 @@ static void zend_rebuild_access_path(zend_cfg *cfg, zend_op_array *op_array, int
 		convert_to_string((v)); \
 	}
 
-static void strip_nop(zend_code_block *block, zend_optimizer_ctx *ctx)
+static void strip_nop(zend_code_block *block, zend_op_array *op_array, zend_optimizer_ctx *ctx)
 {
 	zend_op *opline = block->start_opline;
 	zend_op *end, *new_end;
@@ -575,6 +575,14 @@ static void strip_nop(zend_code_block *block, zend_optimizer_ctx *ctx)
 			}
 			return;
 		}
+		if (block->len == 2
+		 && ((block->start_opline + 1)->opcode == ZEND_JMPZ
+		  || (block->start_opline + 1)->opcode == ZEND_JMPNZ)
+		 && (block->start_opline + 1)->op1_type & (IS_CV|IS_CONST)
+		 && block->start_opline > op_array->opcodes
+		 && zend_is_smart_branch(block->start_opline - 1)) {
+			break;
+		}
 		block->start_opline++;
 		block->start_opline_no++;
 		block->len--;
@@ -588,10 +596,21 @@ static void strip_nop(zend_code_block *block, zend_optimizer_ctx *ctx)
 		zend_op *src;
 		int len = 0;
 
+		src = opline;
 		while (opline < end && opline->opcode == ZEND_NOP) {
+			if (opline + 1 < end
+			 && ((opline + 1)->opcode == ZEND_JMPZ
+			  || (opline + 1)->opcode == ZEND_JMPNZ)
+			 && (opline + 1)->op1_type & (IS_CV|IS_CONST)
+			 && opline > op_array->opcodes
+			 && zend_is_smart_branch(opline - 1)) {
+				/* don't remove NOP, that splits incorrect smart branch */
+				opline++;
+				break;
+			}
+			src++;
 			opline++;
 		}
-		src = opline;
 
 		while (opline < end && opline->opcode != ZEND_NOP) {
 			opline++;
@@ -621,6 +640,14 @@ static void zend_optimize_block(zend_code_block *block, zend_op_array *op_array,
 			if (block->follow_to) {
 				delete_code_block(block, ctx);
 			}
+			if (block->len == 2
+			 && ((block->start_opline + 1)->opcode == ZEND_JMPZ
+			  || (block->start_opline + 1)->opcode == ZEND_JMPNZ)
+			 && (block->start_opline + 1)->op1_type & (IS_CV|IS_CONST)
+			 && block->start_opline > op_array->opcodes
+			 && zend_is_smart_branch(block->start_opline - 1)) {
+				break;
+			}
 			return;
 		}
 		block->start_opline++;
@@ -681,6 +708,9 @@ static void zend_optimize_block(zend_code_block *block, zend_op_array *op_array,
 			MAKE_NOP(src);
 		}
 
+#if 0
+	   /* This pattern is unnecessary for PHP7, 
+		* since compiler won't generate ZEND_FREE for ZEND_BOOL anymore */
        /* T = BOOL(X), FREE(T) => NOP */
 		if (opline->opcode == ZEND_FREE &&
 			ZEND_OP1_TYPE(opline) == IS_TMP_VAR &&
@@ -689,13 +719,15 @@ static void zend_optimize_block(zend_code_block *block, zend_op_array *op_array,
 			if (src->opcode == ZEND_BOOL) {
 				if (ZEND_OP1_TYPE(src) == IS_CONST) {
 					literal_dtor(&ZEND_OP1_LITERAL(src));
+				} else if (ZEND_OP1_TYPE(src) == IS_TMP_VAR) {
+					src->opcode = ZEND_FREE;
+				} else {
+					MAKE_NOP(src);
 				}
-				MAKE_NOP(src);
 				MAKE_NOP(opline);
 			}
 		}
 
-#if 0
 		/* pre-evaluate functions:
 		   constant(x)
 		   defined(x)
@@ -1028,7 +1060,7 @@ static void zend_optimize_block(zend_code_block *block, zend_op_array *op_array,
 				literal_dtor(&ZEND_OP1_LITERAL(opline));
 			} else {
 				/* BOOL */
-				result = ZEND_OP1_LITERAL(opline);
+				ZVAL_COPY_VALUE(&result, &ZEND_OP1_LITERAL(opline));
 				convert_to_boolean(&result);
 				ZVAL_NULL(&ZEND_OP1_LITERAL(opline));
 			}
@@ -1132,7 +1164,7 @@ static void zend_optimize_block(zend_code_block *block, zend_op_array *op_array,
 		opline++;
 	}
 
-	strip_nop(block, ctx);
+	strip_nop(block, op_array, ctx);
 }
 
 /* Rebuild plain (optimized) op_array from CFG */
@@ -1466,9 +1498,9 @@ next_target:
 				    		same_var == VAR_NUM_EX(target->op1) &&
 							target_block->follow_to &&
 							!target_block->protected) {
-					/* JMPZ(X, L), L: X = JMPNZ_EX(X, L2) -> JMPZ(X, L+1) */
+					/* JMPZ(X, L), L: T = JMPNZ_EX(X, L2) -> T = JMPZ_EX(X, L+1) */
 					last_op->opcode += 3;
-					last_op->result = target->result;
+					COPY_NODE(last_op->result, target->result);
 					del_source(block, block->op2_to);
 					block->op2_to = target_block->follow_to;
 					ADD_SOURCE(block, block->op2_to);
@@ -1889,10 +1921,14 @@ static void zend_t_usage(zend_code_block *block, zend_op_array *op_array, zend_b
 					case ZEND_QM_ASSIGN:
 					case ZEND_BOOL:
 					case ZEND_BOOL_NOT:
-						if (ZEND_OP1_TYPE(opline) == IS_CONST) {
-							literal_dtor(&ZEND_OP1_LITERAL(opline));
+						if (ZEND_OP1_TYPE(opline) == IS_TMP_VAR) {
+							opline->opcode = ZEND_FREE;
+						} else {
+							if (ZEND_OP1_TYPE(opline) == IS_CONST) {
+								literal_dtor(&ZEND_OP1_LITERAL(opline));
+							}
+							MAKE_NOP(opline);
 						}
-						MAKE_NOP(opline);
 						break;
 					case ZEND_JMPZ_EX:
 					case ZEND_JMPNZ_EX:
